(define (domain guard)
  (:requirements :strips :typing)
  (:types robot location lift pass-partout person alarm)
  (:predicates (at ?r ?s)
	       (con ?s ?a) 
	       (has-fuel ?r)
	       (in-front-of ?p ?l) 
	       (closed-lift ?l ?p)
     	       (open-lift ?l ?s) 
     	       (open ?p) 
	       (closed ?p)
	       (con-lift ?s ?a)
	       (locked ?p)
	       (not-locked ?p)
	       (controlled ?p)
	       (has-pass-partout ?r ?u)
	       (sensor-on ?p)
	       (sensor-off ?p)
	       (identified ?p)
	       (pass ?p)
	       (not-pass ?p)
	       (alarm ?a)
	       (in ?r ?p))
   

;; 1. operatori di movimento

;; 1.1 operatori per muoversi nel corridoio

  (:action move
	   :parameters (?r - robot ?s - location ?a - location)
	   :precondition (and (at ?r ?s) (con ?s ?a) (has-fuel ?r))
	   :effect (and (at ?r ?a) (not (at ?r ?s))))

;; 1.2 operatori per l'utilizzo dell'ascensore

  (:action call-lift
	   :parameters (?r - robot ?p - location ?l - lift)
	   :precondition (and (at ?r ?p) (in-front-of ?p ?l) (closed-lift ?l ?p))
	   :effect (and (open-lift ?l ?p) (not (closed-lift ?l ?p))))

  (:action take-lift
	   :parameters (?r - robot ?s - location ?a - location ?l - lift)
	   :precondition (and (at ?r ?s) (in-front-of ?s ?l) (in-front-of ?a ?l) (open-lift ?l ?s) (con-lift ?s ?a))
	   :effect (and (at ?r ?a) (closed-lift ?l ?s) (not (at ?r ?s)) (not (open-lift ?l ?s)))) 

;; 1.3 operatori per accedere alle stanze

  (:action open-room
	   :parameters (?r - robot ?l - location ?p - room)
	   :precondition (and (at ?r ?l) (in-front-of ?l ?p) (closed ?p) (sensor-off ?p) (not-locked ?p))
	   :effect (and (open ?p) (not (closed ?p))))

  (:action close-room
	   :parameters (?r - robot ?l - location ?p - room)
	   :precondition (and (at ?r ?l) (in-front-of ?l ?p) (open ?p))
	   :effect (and (closed ?p) (not (open ?p))))

  (:action enter-room
	   :parameters (?r - robot ?l - location ?p - room)
	   :precondition (and (at ?r ?l) (in-front-of ?l ?p) (open ?p))
	   :effect (and (in ?r ?p) (not (at ?r ?l))))
   
  (:action exit-room
	   :parameters (?r - robot ?l - location ?p - room)
	   :precondition (and (in ?r ?p) (in-front-of ?l ?p) (open ?p))
	   :effect (and (at ?r ?l) (not (in ?r ?p))))
   
  (:action control-room
	   :parameters (?r - robot ?p - room)
	   :precondition (and (in ?r ?p))
	   :effect (and (controlled ?p)))


;; 1.4 operatori per sbloccare le stanze e prendere e riportare il pass-partout

  (:action unlock-room
	   :parameters (?r - robot ?l - location ?p - room ?u - pass-partout)
	   :precondition (and (at ?r ?l) (in-front-of ?l ?p) (locked ?p) (has-pass-partout ?r ?u))
	   :effect (and (not-locked ?p) (not (locked ?p))))

  (:action take-pass-partout
	   :parameters (?r - robot ?p - location ?u - pass-partout)
	   :precondition (and (at ?r ?p) (at ?u ?p))
	   :effect (and (has-pass-partout ?r ?u) (not (at ?u ?p))))

  (:action putdown-pass-partout
	   :parameters (?r - robot ?p - location ?u - pass-partout)
	   :precondition (and (at ?r ?p) (has-pass-partout ?r ?u))
	   :effect (and (at ?u ?p) (not (has-pass-partout ?r ?u))))

;; 2. operatori per l'interazione con il centro di controllo

;; 2.1 operatori per la disattivazione dei sensori

  (:action sensor-off
	   :parameters (?r - robot ?l - location ?p - room)
	   :precondition (and (at ?r ?l) (in-front-of ?l ?p) (closed ?p))
	   :effect (and (sensor-off ?p) (not (sensor-on ?p))))

  (:action sensor-on
	   :parameters (?r - robot ?l - location ?p - room)
	   :precondition (and (at ?r ?l) (in-front-of ?l ?p) (closed ?p))
	   :effect (and (sensor-on ?p) (not (sensor-off ?p))))

;; 3. operatori per l'interazione con le persone

  (:action person-ok
	   :parameters (?r - robot ?l - location ?p - person)
	   :precondition (and (at ?r ?l) (at ?p ?l) (pass ?p))
	   :effect (and (identified ?p)))
  
  (:action alarm-person
	   :parameters (?r - robot ?l - location ?p - person ?a - alarm)
	   :precondition (and (at ?r ?l) (at ?p ?l) (not-pass ?p) )
	   :effect (and (identified ?p) (alarm ?a))))
  

