(define (domain room)
  (:requirements :strips :typing)
  (:types robot location alarm light object precious-object window safe-box)
  (:predicates (at ?r ?s) 
               (con ?s ?a) 
	       (has-fuel ?r) 
	       (image-on ?r)
	       (image-off ?r)
	       (on ?l) 
	       (off ?l) 
	       (present ?o ?l)
	       (not-present ?o ?l)
	       (controlled ?o)
	       (alarm-on ?a)
	       (present-complete ?p ?l)
	       (not-present-complete ?p ?l)
	       (open ?w)
	       (closed ?w)
	       (locked ?s)
	       (not-locked ?s))
	       
;; mondo costituito da una stanza da controllare

;; 1. operatori di movimento

   (:action move
	    :parameters (?r - robot ?s - location ?a - location)
	    :precondition (and (at ?r ?s) (con ?s ?a) (has-fuel ?r) (image-on ?r))
	    :effect (and (at ?r ?a) (not (at ?r ?s))))
   
;; 2. operatori per accendere e spegnere luci e telecamera

   (:action light-image-on
	    :parameters (?l - light ?r - robot)
	    :precondition (and (off ?l))
	    :effect (and (on ?l) (image-on ?r) (not (off ?l)) (not (image-off ?r))))
   
   (:action send-image
	    :parameters (?l - light ?r - robot)
	    :precondition (on ?l)
	    :effect (and (image-on ?r) (not (image-off ?r))))
   
   (:action light-image-off
	    :parameters (?l - light ?r - robot)
	    :precondition (and (on ?l) (image-on ?r))
	    :effect (and (off ?l) (image-off ?r) (not (on ?l)) (not (image-on ?r))))


;; 3. operatori per il controllo della presenza di oggetti

   (:action control-object
	    :parameters (?o - object ?r - robot ?l - location)
	    :precondition (and (present ?o ?l) (at ?r ?l) (image-on ?r))
	    :effect (and (controlled ?o)))

   (:action alarm-object
	    :parameters (?o - object ?r - robot ?l - location ?a - alarm)
	    :precondition (and (not-present ?o ?l) (at ?r ?l) (image-on ?r))
	    :effect (and (controlled ?o) (alarm-on ?a)))

;; 4. operatori per il controllo dell'integrita' degli oggetti preziosi

   (:action control-precious-object
	    :parameters (?p - precious-object ?r - robot ?l - location)
	    :precondition (and (at ?r ?l) (present-complete ?p ?l) (image-on ?r))
	    :effect (and (controlled ?p)))

   (:action alarm-precious-object
	    :parameters (?p - precious-object ?r - robot ?l - location ?a - alarm)
	    :precondition (and (at ?r ?l) (not-present-complete ?p ?l) (image-on ?r))
	    :effect (and (controlled ?p) (alarm-on ?a)))

;; 5. operatori per il controllo delle finestre

   (:action control-window
	    :parameters (?w - window ?r - robot ?l - location)
	    :precondition (and (at ?w ?l) (at ?r ?l) (closed ?w) (image-on ?r))
	    :effect (and (controlled ?w)))

   (:action alarm-window
	    :parameters (?w - window ?r - robot ?l - location ?a - alarm)
	    :precondition (and (at ?w ?l) (at ?r ?l) (open ?w) (image-on ?r))
	    :effect (and (controlled ?w) (alarm-on ?a)))

;; 6. operatori per il controllo della cassaforte

   (:action control-safe-box
	    :parameters (?s - safe-box ?r - robot ?l - location)
	    :precondition (and (at ?s ?l) (at ?r ?l) (locked ?s) (image-on ?r))
	    :effect (and (controlled ?s)))

   (:action alarm-safe-box
	    :parameters (?s - safe-box ?r - robot ?l - location ?a - alarm)
	    :precondition (and (at ?s ?l) (at ?r ?l) (not-locked ?s) (image-on ?r))
	    :effect (and (controlled ?s) (alarm-on ?a))))